@@ -26,16 +26,34 @@
 #'   domain = 'substance'
 #' )
 get_all_sources <- function(domain = 'substance') {
-  # Use the get function to retrieve the sources
-  response_content <- get_pubchem(identifier = domain, namespace = NULL, domain = 'sources')
+  # Validate the domain parameter
+  if (!domain %in% c("substance", "compound", "assay")) {
+    stop("Invalid domain specified. Please choose from 'substance', 'compound', or 'assay'.")
+  }
 
-  # Convert the response content to a list using fromJSON
-  results <- fromJSON(response_content)
+  # Attempt to retrieve the sources using the get_pubchem function
+  response_content <- tryCatch({
+    get_pubchem(identifier = domain, namespace = NULL, domain = 'sources')
+  }, error = function(e) {
+    stop(paste("Failed to retrieve sources for the specified domain:", conditionMessage(e)))
+  })
 
-  # Return the SourceName from the results
+  # Validate the response content
+  if (is.null(response_content) || nchar(response_content) == 0) {
+    stop("Error: No response content received from the PubChem server.")
+  }
+
+  # Attempt to parse the JSON response content
+  results <- tryCatch({
+    fromJSON(response_content)
+  }, error = function(e) {
+    stop(paste("Failed to parse JSON response:", conditionMessage(e)))
+  })
+
+  # Check if 'SourceName' exists in the parsed response
   if ("InformationList" %in% names(results) && "SourceName" %in% names(results$InformationList)) {
     return(results$InformationList$SourceName)
   } else {
-    stop("Error: SourceName not found in the response.")
+    stop("Error: 'SourceName' not found in the response. Check the response structure.")
   }
 }
